@@ -9,13 +9,8 @@ This leverages the TemporalExecutor's start_worker method to handle the worker s
 import asyncio
 import logging
 
+import workflows  # noqa: F401
 from main import app
-from basic import SimpleWorkflow  # noqa: F401
-from evaluator_optimizer import EvaluatorOptimizerWorkflow  # noqa: F401
-from orchestrator import OrchestratorWorkflow  # noqa: F401
-from parallel import ParallelWorkflow  # noqa: F401
-from router import RouterWorkflow  # noqa: F401
-from interactive import WorkflowWithInteraction  # noqa: F401
 
 from mcp_agent.executor.temporal import create_temporal_worker_for_app
 
