@@ -709,10 +709,9 @@ class Agent(BaseModel):
                         f"Filtered tools: {[name for name, _ in filtered_out_tools[:10]]}"
                         + ("..." if len(filtered_out_tools) > 10 else "")
                     )
-                    # Log detailed reasons at trace level (if trace logging is available)
-                    if logger.isEnabledFor(10):  # TRACE level is usually 10
-                        for tool_name, reason in filtered_out_tools:
-                            logger.log(10, f"Filtered out '{tool_name}': {reason}")
+
+                    for tool_name, reason in filtered_out_tools:
+                        logger.debug(f"Filtered out '{tool_name}': {reason}")
                 else:
                     logger.debug(
                         f"Tool filter applied: All {len(result.tools)} tools passed the filter"
