@@ -1,5 +1,6 @@
 """MCP Agent Cloud apps command."""
 
 from .list import list_apps
+from .update import update_app
 
-__all__ = ["list_apps"]
+__all__ = ["list_apps", "update_app"]
