@@ -1,3 +1,4 @@
+import json
 import os
 import re
 import shutil
@@ -5,22 +6,31 @@ import subprocess
 import tempfile
 import textwrap
 from pathlib import Path
-import json
 
 from rich.progress import Progress, SpinnerColumn, TextColumn
 
 from mcp_agent.cli.config import settings
 from mcp_agent.cli.core.constants import MCP_SECRETS_FILENAME
-from mcp_agent.cli.utils.ux import console, print_error, print_warning, print_info
 from mcp_agent.cli.utils.git_utils import (
     get_git_metadata,
     compute_directory_fingerprint,
     utc_iso_now,
 )
-
+from mcp_agent.cli.utils.ux import (
+    console,
+    print_error,
+    print_warning,
+    print_info,
+    print_verbose,
+)
+from .bundle_utils import (
+    create_pathspec_from_gitignore,
+    should_ignore_by_gitignore,
+)
 from .constants import (
     CLOUDFLARE_ACCOUNT_ID,
     CLOUDFLARE_EMAIL,
+    DEFAULT_DEPLOYMENTS_UPLOAD_API_BASE_URL,
     WRANGLER_SEND_METRICS,
 )
 from .settings import deployment_settings
@@ -107,7 +117,12 @@ def _handle_wrangler_error(e: subprocess.CalledProcessError) -> None:
         print_error(clean_output)
 
 
-def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
+def wrangler_deploy(
+    app_id: str,
+    api_key: str,
+    project_dir: Path,
+    ignore_file: Path | None = None,
+) -> None:
     """Bundle the MCP Agent using Wrangler.
 
     A thin wrapper around the Wrangler CLI to bundle the MCP Agent application code
@@ -128,6 +143,7 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
         app_id (str): The application ID.
         api_key (str): User MCP Agent Cloud API key.
         project_dir (Path): The directory of the project to deploy.
+        ignore_file (Path | None): Optional path to a gitignore-style file for excluding files from the bundle.
     """
 
     # Copy existing env to avoid overwriting
@@ -154,6 +170,32 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
         npm_prefix.mkdir(parents=True, exist_ok=True)
         env_updates["npm_config_prefix"] = str(npm_prefix)
 
+    if os.environ.get("__MCP_DISABLE_TLS_VALIDATION", "").lower() in (
+        "1",
+        "true",
+        "yes",
+    ):
+        if (
+            deployment_settings.DEPLOYMENTS_UPLOAD_API_BASE_URL
+            == DEFAULT_DEPLOYMENTS_UPLOAD_API_BASE_URL
+        ):
+            print_error(
+                f"Cannot disable TLS validation when using {DEFAULT_DEPLOYMENTS_UPLOAD_API_BASE_URL}. "
+                "Set MCP_DEPLOYMENTS_UPLOAD_API_BASE_URL to a custom endpoint."
+            )
+            raise ValueError(
+                f"TLS validation cannot be disabled with {DEFAULT_DEPLOYMENTS_UPLOAD_API_BASE_URL}"
+            )
+
+        env_updates["NODE_TLS_REJECT_UNAUTHORIZED"] = "0"
+        print_warning(
+            "TLS certificate validation disabled (__MCP_DISABLE_TLS_VALIDATION is set)."
+        )
+        if settings.VERBOSE:
+            print_info(
+                f"Deployment endpoint: {deployment_settings.DEPLOYMENTS_UPLOAD_API_BASE_URL}"
+            )
+
     env.update(env_updates)
 
     validate_project(project_dir)
@@ -165,18 +207,41 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
     with tempfile.TemporaryDirectory(prefix="mcp-deploy-") as temp_dir_str:
         temp_project_dir = Path(temp_dir_str) / "project"
 
-        # Copy the entire project to temp directory, excluding unwanted directories and secrets file
-        def ignore_patterns(_path, names):
+        # Load ignore rules (gitignore syntax) only if an explicit ignore file is provided
+        ignore_spec = (
+            create_pathspec_from_gitignore(ignore_file) if ignore_file else None
+        )
+        if ignore_file:
+            if ignore_spec is None:
+                print_warning(
+                    f"Ignore file '{ignore_file}' not found; applying default excludes only"
+                )
+            else:
+                print_info(f"Using ignore patterns from {ignore_file}")
+        else:
+            print_verbose("No ignore file provided; applying default excludes only")
+
+        # Copy the entire project to temp directory, excluding unwanted directories and the live secrets file
+        def ignore_patterns(path_str, names):
             ignored = set()
+
+            # Keep existing hardcoded exclusions (highest priority)
             for name in names:
                 if (name.startswith(".") and name not in {".env"}) or name in {
                     "logs",
                     "__pycache__",
                     "node_modules",
                     "venv",
-                    MCP_SECRETS_FILENAME,
+                    MCP_SECRETS_FILENAME,  # Exclude mcp_agent.secrets.yaml only
                 }:
                     ignored.add(name)
+
+            # Apply explicit ignore file patterns (if provided)
+            spec_ignored = should_ignore_by_gitignore(
+                path_str, names, project_dir, ignore_spec
+            )
+            ignored.update(spec_ignored)
+
             return ignored
 
         shutil.copytree(project_dir, temp_project_dir, ignore=ignore_patterns)
@@ -209,6 +274,29 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
                 # Rename in place
                 file_path.rename(py_path)
 
+        # Compute and log which original files are being bundled (skip internal helpers)
+        bundled_original_files: list[str] = []
+        internal_bundle_files = {"wrangler.toml", "mcp_deploy_breadcrumb.py"}
+        for root, _dirs, files in os.walk(temp_project_dir):
+            for filename in files:
+                rel = Path(root).relative_to(temp_project_dir) / filename
+                if filename in internal_bundle_files:
+                    continue
+                if filename.endswith(".mcpac.py"):
+                    orig_rel = str(rel)[: -len(".mcpac.py")]
+                    bundled_original_files.append(orig_rel)
+                else:
+                    bundled_original_files.append(str(rel))
+
+        bundled_original_files.sort()
+        if bundled_original_files:
+            print_verbose(
+                "\n".join(
+                    [f"Bundling {len(bundled_original_files)} project file(s):"]
+                    + [f" - {p}" for p in bundled_original_files]
+                )
+            )
+
         # Collect deployment metadata (git if available, else workspace hash)
         git_meta = get_git_metadata(project_dir)
         deploy_source = "git" if git_meta else "workspace"
@@ -244,7 +332,9 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
                 },
             )
             meta_vars.update({"MCP_DEPLOY_WORKSPACE_HASH": bundle_hash})
-            print_info(f"Deploying from non-git workspace (hash {bundle_hash[:12]}…)")
+            print_verbose(
+                f"Deploying from non-git workspace (hash {bundle_hash[:12]}…)"
+            )
 
         # Write a breadcrumb file into the project so it ships with the bundle.
         # Use a Python file for guaranteed inclusion without renaming.
@@ -305,9 +395,11 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
         wrangler_toml_path = temp_project_dir / "wrangler.toml"
         wrangler_toml_path.write_text(wrangler_toml_content)
 
+        spinner_column = SpinnerColumn(spinner_name="aesthetic")
         with Progress(
-            SpinnerColumn(spinner_name="aesthetic"),
-            TextColumn("[progress.description]{task.description}"),
+            "",
+            spinner_column,
+            TextColumn(" [progress.description]{task.description}"),
         ) as progress:
             task = progress.add_task("Bundling MCP Agent...", total=None)
 
@@ -335,9 +427,8 @@ def wrangler_deploy(app_id: str, api_key: str, project_dir: Path) -> None:
                     encoding="utf-8",
                     errors="replace",
                 )
-                progress.update(task, description="✅ Bundled successfully")
-                return
-
+                spinner_column.spinner.frames = spinner_column.spinner.frames[-2:-1]
+                progress.update(task, description="Bundled successfully")
             except subprocess.CalledProcessError as e:
                 progress.update(task, description="❌ Bundling failed")
                 _handle_wrangler_error(e)
