@@ -18,6 +18,7 @@ from rich.table import Table
 from rich.prompt import Prompt, Confirm
 from rich.progress import Progress, SpinnerColumn, TextColumn
 
+from mcp_agent.cli.utils.ux import LOG_VERBOSE
 
 app = typer.Typer(help="Manage provider API keys")
 console = Console()
@@ -172,6 +173,10 @@ def show(
     """Show configured API keys and their status."""
     from mcp_agent.config import get_settings
 
+    if verbose:
+        LOG_VERBOSE.set(True)
+    verbose = LOG_VERBOSE.get()
+
     console.print("\n[bold cyan]🔑 API Key Status[/bold cyan]\n")
 
     settings = get_settings()
@@ -451,6 +456,10 @@ def test(
 
     console.print("\n[bold cyan]🧪 Testing API Keys[/bold cyan]\n")
 
+    if verbose:
+        LOG_VERBOSE.set(True)
+    verbose = LOG_VERBOSE.get()
+
     settings = get_settings()
 
     # Determine which providers to test
