@@ -19,12 +19,12 @@ from rich.live import Live
 from rich.progress import Progress, SpinnerColumn, TextColumn
 
 from mcp_agent.server.app_server import create_mcp_server_for_app
-from mcp_agent.cli.core.utils import load_user_app
+from mcp_agent.cli.core.utils import load_user_app, detect_default_script
 from mcp_agent.config import get_settings
 
 
 app = typer.Typer(help="Serve app as an MCP server")
-console = Console()
+console = Console(stderr=True)
 
 
 class ServerMonitor:
@@ -118,9 +118,9 @@ def serve(
     Start an MCP server for your app.
 
     Examples:
-        mcp-agent serve --script agent.py
-        mcp-agent serve --transport http --port 8000
-        mcp-agent serve --reload --debug
+        mcp-agent dev serve --script agent.py
+        mcp-agent dev serve --transport http --port 8000
+        mcp-agent dev serve --reload --debug
     """
 
     if ctx.invoked_subcommand:
@@ -138,19 +138,37 @@ def serve(
     # Load configuration path is handled after loading app by overriding app settings
 
     async def _run():
-        # Load the app
-        script_path = Path(script) if script else Path("agent.py")
+        # Load the app (auto-detect main.py preferred)
+        script_path = detect_default_script(Path(script) if script else None)
 
         if not script_path.exists():
             console.print(f"[red]Script not found: {script_path}[/red]")
-            console.print("\n[dim]Create an agent.py file or specify --script[/dim]")
+            console.print(
+                "\n[dim]Create a main.py (preferred) or agent.py file, or specify --script[/dim]"
+            )
             raise typer.Exit(1)
 
         console.print("\n[bold cyan]🚀 MCP-Agent Server[/bold cyan]")
         console.print(f"Script: [green]{script_path}[/green]")
 
+        # Load settings from config if provided
+        settings_override = None
+        if config:
+            try:
+                from mcp_agent.config import get_settings as _get_settings
+
+                settings_override = _get_settings(config_path=str(config))
+                console.print(f"Config: [green]{config}[/green]")
+            except Exception as _e:
+                console.print(f"[red]Failed to load config: {_e}[/red]")
+                if debug:
+                    import traceback
+
+                    console.print(f"[dim]{traceback.format_exc()}[/dim]")
+                raise typer.Exit(1)
+
         try:
-            app_obj = load_user_app(script_path)
+            app_obj = load_user_app(script_path, settings_override=settings_override)
         except Exception as e:
             console.print(f"[red]Failed to load app: {e}[/red]")
             if debug:
@@ -158,18 +176,6 @@ def serve(
 
                 console.print(f"[dim]{traceback.format_exc()}[/dim]")
             raise typer.Exit(1)
-
-        # Override app settings if a config path is provided
-        if config:
-            try:
-                from mcp_agent.config import get_settings as _get_settings
-
-                app_obj._config = _get_settings(config_path=str(config))  # type: ignore[attr-defined]
-            except Exception as _e:
-                if debug:
-                    console.print(
-                        f"[yellow]Warning: failed to apply config override: {_e}[/yellow]"
-                    )
         # Initialize the app
         await app_obj.initialize()
 
@@ -379,10 +385,13 @@ def test(
     timeout: float = typer.Option(5.0, "--timeout", "-t", help="Test timeout"),
 ) -> None:
     """Test if the server can be loaded and initialized."""
-    script_path = Path(script) if script else Path("agent.py")
+    script_path = detect_default_script(Path(script) if script else None)
 
     if not script_path.exists():
         console.print(f"[red]Script not found: {script_path}[/red]")
+        console.print(
+            "\n[dim]Create a main.py (preferred) or agent.py file, or specify --script[/dim]"
+        )
         raise typer.Exit(1)
 
     console.print(f"\n[bold]Testing server: {script_path}[/bold]\n")
@@ -463,7 +472,7 @@ def test(
             )
 
             console.print("\n[dim]Server is ready to run with:[/dim]")
-            console.print(f"  [cyan]mcp-agent serve --script {script_path}[/cyan]")
+            console.print(f"  [cyan]mcp-agent dev serve --script {script_path}[/cyan]")
 
         except Exception:
             console.print("\n[red bold]❌ Server test failed[/red bold]")
@@ -526,9 +535,11 @@ def generate(
     console.print("\n[bold]Next steps:[/bold]")
     console.print(f"1. Edit the server: [cyan]{output}[/cyan]")
     console.print(
-        f"2. Test the server: [cyan]mcp-agent serve test --script {output}[/cyan]"
+        f"2. Test the server: [cyan]mcp-agent dev serve test --script {output}[/cyan]"
     )
-    console.print(f"3. Run the server: [cyan]mcp-agent serve --script {output}[/cyan]")
     console.print(
-        f"4. Or serve via HTTP: [cyan]mcp-agent serve --script {output} --transport http --port 8000[/cyan]"
+        f"3. Run the server: [cyan]mcp-agent dev serve --script {output}[/cyan]"
+    )
+    console.print(
+        f"4. Or serve via HTTP: [cyan]mcp-agent dev serve --script {output} --transport http --port 8000[/cyan]"
     )
