@@ -218,6 +218,7 @@ class Workflow(ABC, Generic[T], ContextDependent):
 
         import asyncio
         from concurrent.futures import CancelledError
+        import traceback
 
         handle: "WorkflowHandle" | None = None
 
@@ -315,7 +316,11 @@ class Workflow(ABC, Generic[T], ContextDependent):
                         return result
 
                 except Exception as e:
-                    self._logger.error(f"Error waiting for tasks: {e}")
+                    self._logger.error(
+                        "Error waiting for tasks",
+                        exception=repr(e),
+                        traceback=traceback.format_exc(),
+                    )
                     raise
 
             except CancelledError:
@@ -761,9 +766,23 @@ class Workflow(ABC, Generic[T], ContextDependent):
                 if isinstance(memo_map, dict):
                     gateway_url = memo_map.get("gateway_url")
                     gateway_token = memo_map.get("gateway_token")
+                    sanitized_token = None
+                    if isinstance(gateway_token, str):
+                        # If it's an MCP API key, include some suffix to allow debugging
+                        if (
+                            gateway_token.startswith("lm_mcp_api_")
+                            and len(gateway_token) > 24
+                        ):
+                            sanitized_token = (
+                                f"{gateway_token[:10]}...{gateway_token[-4:]}"
+                            )
+                        elif len(gateway_token) > 10:
+                            sanitized_token = f"{gateway_token[:4]}..."
+                        else:
+                            sanitized_token = "***"
 
                     self._logger.debug(
-                        f"Proxy parameters: gateway_url={gateway_url}, gateway_token={gateway_token}"
+                        f"Proxy parameters: gateway_url={gateway_url}, gateway_token={sanitized_token}"
                     )
 
                     if gateway_url:
@@ -788,10 +807,11 @@ class Workflow(ABC, Generic[T], ContextDependent):
                 upstream_session = getattr(self.context, "upstream_session", None)
 
                 if upstream_session is None:
-                    self.context.upstream_session = SessionProxy(
+                    proxy_session = SessionProxy(
                         executor=self.executor,
                         context=self.context,
                     )
+                    self.context.upstream_session = proxy_session
 
                     app = self.context.app
                     if app:
