@@ -8,6 +8,7 @@ import asyncio
 import json
 import uuid
 import datetime
+import sys
 from abc import ABC, abstractmethod
 from typing import Dict, List, Protocol
 from pathlib import Path
@@ -325,7 +326,15 @@ class AsyncEventBus:
             # Signal shutdown
             cls._instance._running = False
             if hasattr(cls._instance, "_stop_event"):
-                cls._instance._stop_event.set()
+                try:
+                    # _stop_event.set() schedules on the event's loop; this can fail if
+                    # the loop is already closed in test teardown. Swallow to ensure
+                    # reset never raises in those cases.
+                    cls._instance._stop_event.set()
+                except RuntimeError:
+                    pass
+                except Exception:
+                    pass
 
             # Clear the singleton instance
             cls._instance = None
@@ -424,6 +433,22 @@ class AsyncEventBus:
         # Then queue for listeners
         await self._queue.put(event)
 
+    def emit_with_stderr_transport(self, event: Event):
+        print(
+            f"[{event.type}] {event.namespace}: {event.message}",
+            file=sys.stderr,
+        )
+
+        # Initialize queue and start processing if needed
+        if not hasattr(self, "_queue"):
+            self.init_queue()
+            # Auto-start the event processing task if not running
+            if not self._running:
+                self._running = True
+                self._task = asyncio.create_task(self._process_events())
+
+        self._queue.put_nowait(event)
+
     async def _send_to_transport(self, event: Event):
         """Send event to transport with error handling."""
         try:
