@@ -4,7 +4,17 @@ from typing import Any, Iterable, List, Type, Union, cast
 
 from pydantic import BaseModel
 
-from anthropic import Anthropic, AnthropicBedrock, AnthropicVertex, AsyncAnthropic
+from anthropic import (
+    Anthropic,
+    AnthropicBedrock,
+    AnthropicVertex,
+    AsyncAnthropic,
+    AuthenticationError,
+    BadRequestError,
+    NotFoundError,
+    PermissionDeniedError,
+    UnprocessableEntityError,
+)
 from anthropic.types import (
     ContentBlock,
     DocumentBlockParam,
@@ -38,6 +48,7 @@ from mcp.types import (
 # from mcp_agent.agents.agent import HUMAN_INPUT_TOOL_NAME
 from mcp_agent.config import AnthropicSettings
 from mcp_agent.executor.workflow_task import workflow_task
+from mcp_agent.executor.errors import to_application_error
 from mcp_agent.tracing.semconv import (
     GEN_AI_AGENT_NAME,
     GEN_AI_REQUEST_MODEL,
@@ -61,6 +72,14 @@ from mcp_agent.workflows.llm.augmented_llm import (
 from mcp_agent.logging.logger import get_logger
 from mcp_agent.workflows.llm.multipart_converter_anthropic import AnthropicConverter
 
+_NON_RETRYABLE_ANTHROPIC_ERRORS = (
+    AuthenticationError,
+    PermissionDeniedError,
+    BadRequestError,
+    NotFoundError,
+    UnprocessableEntityError,
+)
+
 MessageParamContent = Union[
     str,
     Iterable[
@@ -102,6 +121,13 @@ def create_anthropic_instance(settings: AnthropicSettings):
     return anthropic
 
 
+async def _execute_anthropic_async(client: AsyncAnthropic, payload: dict) -> Message:
+    try:
+        return await client.messages.create(**payload)
+    except _NON_RETRYABLE_ANTHROPIC_ERRORS as exc:
+        raise to_application_error(exc, non_retryable=True) from exc
+
+
 class AnthropicAugmentedLLM(AugmentedLLM[MessageParam, Message]):
     """
     The basic building block of agentic systems is an LLM enhanced with augmentations
@@ -475,8 +501,17 @@ class AnthropicAugmentedLLM(AugmentedLLM[MessageParam, Message]):
                 client = AsyncAnthropic()
 
             async with client:
-                async with client.messages.stream(**args) as stream:
-                    final = await stream.get_final_message()
+                stream_method = client.messages.stream
+                if all(
+                    hasattr(stream_method, attr) for attr in ("__aenter__", "__aexit__")
+                ):
+                    async with stream_method(**args) as stream:
+                        final = await stream.get_final_message()
+                else:
+                    # The OpenTelemetry anthropic instrumentation wraps stream() and
+                    # returns an async generator that is not an async context manager.
+                    # Fallback to create() so the call succeeds while still emitting spans.
+                    final = await client.messages.create(**args)
 
             # Extract tool_use input and validate
             for block in final.content:
@@ -760,7 +795,7 @@ class AnthropicAugmentedLLM(AugmentedLLM[MessageParam, Message]):
 
 class AnthropicCompletionTasks:
     @staticmethod
-    @workflow_task
+    @workflow_task(retry_policy={"maximum_attempts": 3})
     @telemetry.traced()
     async def request_completion_task(
         request: RequestCompletionRequest,
@@ -768,22 +803,23 @@ class AnthropicCompletionTasks:
         """
         Request a completion from Anthropic's API.
         """
-        # Prefer async client where available to avoid blocking the event loop
+        payload = request.payload
+
         if request.config.provider in (None, "", "anthropic"):
             client = AsyncAnthropic(api_key=request.config.api_key)
-            payload = request.payload
-            response = await client.messages.create(**payload)
-            response = ensure_serializable(response)
-            return response
+            response = await _execute_anthropic_async(client, payload)
         else:
             anthropic = create_anthropic_instance(request.config)
-            payload = request.payload
             loop = asyncio.get_running_loop()
-            response = await loop.run_in_executor(
-                None, functools.partial(anthropic.messages.create, **payload)
-            )
-            response = ensure_serializable(response)
-            return response
+            try:
+                response = await loop.run_in_executor(
+                    None, functools.partial(anthropic.messages.create, **payload)
+                )
+            except _NON_RETRYABLE_ANTHROPIC_ERRORS as exc:
+                raise to_application_error(exc, non_retryable=True) from exc
+
+        response = ensure_serializable(response)
+        return response
 
 
 class AnthropicMCPTypeConverter(ProviderToMCPConverter[MessageParam, Message]):
