@@ -30,6 +30,16 @@ from opentelemetry import trace
 
 from pydantic import BaseModel
 
+from openai import (
+    AsyncAzureOpenAI,
+    AuthenticationError as AzureOpenAIAuthenticationError,
+    BadRequestError as AzureOpenAIBadRequestError,
+    NotFoundError as AzureOpenAINotFoundError,
+    PermissionDeniedError as AzureOpenAIPermissionDeniedError,
+    UnprocessableEntityError as AzureOpenAIUnprocessableEntityError,
+)
+from openai.types.chat import ChatCompletion
+
 from mcp.types import (
     CallToolRequestParams,
     CallToolRequest,
@@ -62,6 +72,17 @@ from mcp_agent.workflows.llm.augmented_llm import (
 )
 from mcp_agent.logging.logger import get_logger
 from mcp_agent.workflows.llm.multipart_converter_azure import AzureConverter
+from mcp_agent.executor.errors import to_application_error
+
+_NON_RETRYABLE_AZURE_STATUS_CODES = {400, 401, 403, 404, 422}
+
+_NON_RETRYABLE_AZURE_OPENAI_ERRORS = (
+    AzureOpenAIAuthenticationError,
+    AzureOpenAIPermissionDeniedError,
+    AzureOpenAIBadRequestError,
+    AzureOpenAINotFoundError,
+    AzureOpenAIUnprocessableEntityError,
+)
 
 MessageParam = Union[
     SystemMessage, UserMessage, AssistantMessage, ToolMessage, DeveloperMessage
@@ -106,6 +127,8 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
         # Get default model from config if available
         default_model = "gpt-4o-mini"  # Fallback default
 
+        self._is_openai_model = lambda model: model and model.lower().startswith("gpt-")
+
         if self.context.config.azure:
             if hasattr(self.context.config.azure, "default_model"):
                 default_model = self.context.config.azure.default_model
@@ -217,10 +240,19 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
                 )
                 self._annotate_span_for_completion_request(span, request, i)
 
-                response = await self.executor.execute(
-                    AzureCompletionTasks.request_completion_task,
-                    request,
-                )
+                # Route to appropriate completion task based on model type
+                if self._is_openai_model(model):
+                    # Use OpenAI client for GPT models
+                    response = await self.executor.execute(
+                        AzureOpenAICompletionTasks.request_completion_task,
+                        request,
+                    )
+                else:
+                    # Use Azure AI Inference client for non-GPT models
+                    response = await self.executor.execute(
+                        AzureCompletionTasks.request_completion_task,
+                        request,
+                    )
 
                 if isinstance(response, BaseException):
                     self.logger.error(f"Error: {response}")
@@ -233,8 +265,12 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
                 self._annotate_span_for_completion_response(span, response, i)
 
                 # Per-iteration token counts
-                iteration_input = response.usage["prompt_tokens"]
-                iteration_output = response.usage["completion_tokens"]
+                if isinstance(response.usage, dict):
+                    iteration_input = response.usage["prompt_tokens"]
+                    iteration_output = response.usage["completion_tokens"]
+                else:
+                    iteration_input = response.usage.prompt_tokens
+                    iteration_output = response.usage.completion_tokens
 
                 total_input_tokens += iteration_input
                 total_output_tokens += iteration_output
@@ -520,14 +556,27 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
         return attrs
 
 
+def _raise_non_retryable_azure(
+    error: Exception, status_code: int | None = None
+) -> None:
+    message = str(error)
+    if status_code is not None:
+        message = f"{status_code}: {message}"
+    raise to_application_error(
+        error,
+        message=message,
+        non_retryable=True,
+    ) from error
+
+
 class AzureCompletionTasks:
     @staticmethod
-    @workflow_task
+    @workflow_task(retry_policy={"maximum_attempts": 3})
     async def request_completion_task(
         request: RequestCompletionRequest,
     ) -> ChatCompletions:
         """
-        Request a completion from Azure's API.
+        Request a completion from Azure's API using Azure AI Inference.
         """
         if request.config.api_key:
             azure_client = ChatCompletionsClient(
@@ -555,30 +604,96 @@ class AzureCompletionTasks:
         except HttpResponseError as e:
             logger = get_logger(__name__)
 
-            if e.status_code != 400:
-                logger.error(f"Azure API call failed: {e}")
-                raise
-
-            logger.warning(
-                f"Initial Azure API call failed: {e}. Retrying with fallback parameters."
-            )
-
-            # Create a new payload with fallback values for commonly problematic parameters
-            fallback_payload = {**payload, "max_tokens": None, "temperature": 1}
-
-            try:
-                response = await loop.run_in_executor(
-                    None, functools.partial(azure_client.complete, **fallback_payload)
+            if e.status_code == 400:
+                logger.warning(
+                    "Initial Azure API call failed with status 400; retrying with fallback parameters."
                 )
-            except Exception as retry_error:
-                # If retry also fails, raise a more informative error
-                raise RuntimeError(
-                    f"Azure API call failed even with fallback parameters. "
-                    f"Original error: {e}. Retry error: {retry_error}"
-                ) from retry_error
+                fallback_payload = {**payload, "max_tokens": None, "temperature": 1}
+                try:
+                    response = await loop.run_in_executor(
+                        None,
+                        functools.partial(azure_client.complete, **fallback_payload),
+                    )
+                except HttpResponseError as retry_error:
+                    if retry_error.status_code in _NON_RETRYABLE_AZURE_STATUS_CODES:
+                        _raise_non_retryable_azure(retry_error, retry_error.status_code)
+                    raise
+                except Exception as retry_error:
+                    _raise_non_retryable_azure(retry_error)
+            elif e.status_code in _NON_RETRYABLE_AZURE_STATUS_CODES:
+                _raise_non_retryable_azure(e, e.status_code)
+            else:
+                logger.error("Azure API call failed: %s", e)
+                raise
         return response
 
 
+class AzureOpenAICompletionTasks:
+    @staticmethod
+    @workflow_task(retry_policy={"maximum_attempts": 3})
+    async def request_completion_task(
+        request: RequestCompletionRequest,
+    ) -> ChatCompletion:
+        """
+        Request a completion from Azure OpenAI API using the openai library.
+        This is used for GPT models on Azure.
+        """
+
+        def _openai_reasoning(model: str):
+            return model and model.startswith(("gpt-5", "gpt-o1", "gpt-o3", "gpt-o4"))
+
+        payload = request.payload.copy()
+
+        # Handle reasoning models
+        if _openai_reasoning(payload.get("model")):
+            # Newer reasoning models use 'max_completion_tokens' instead of 'max_tokens'
+            max_tokens = payload.get("max_tokens")
+            if max_tokens:
+                payload["max_completion_tokens"] = max_tokens
+                del payload["max_tokens"]
+
+            # Remove parameters that reasoning models don't support
+            params_to_remove = [
+                "temperature",
+                "top_p",
+                "presence_penalty",
+                "frequency_penalty",
+            ]
+            for param in params_to_remove:
+                payload.pop(param, None)
+
+        # Build client parameters
+        client_params = {
+            "azure_endpoint": request.config.endpoint,
+            "api_version": request.config.api_version,
+        }
+
+        # Handle authentication - prioritize API key, then Azure AD token, then Azure AD token provider
+        if request.config.api_key:
+            client_params["api_key"] = request.config.api_key
+        elif request.config.azure_ad_token:
+            client_params["azure_ad_token"] = request.config.azure_ad_token
+        elif request.config.azure_ad_token_provider:
+            client_params["azure_ad_token_provider"] = (
+                request.config.azure_ad_token_provider
+            )
+        else:
+            # Fall back to API key from environment if available
+            client_params["api_key"] = request.config.api_key
+
+        async with AsyncAzureOpenAI(**client_params) as client:
+            # Azure deployment name: use azure_deployment from config if specified,
+            # otherwise use the model name as deployment name
+            deployment = request.config.azure_deployment or payload.get("model")
+            payload["model"] = deployment
+            try:
+                response = await client.chat.completions.create(**payload)
+            except _NON_RETRYABLE_AZURE_OPENAI_ERRORS as exc:
+                _raise_non_retryable_azure(exc)
+
+            return response
+
+
 class MCPAzureTypeConverter(ProviderToMCPConverter[MessageParam, ResponseMessage]):
     """
     Convert between Azure and MCP types.
